/* ====================================================================
   Licensed to the Apache Software Foundation (ASF) under one or more
   contributor license agreements.  See the NOTICE file distributed with
   this work for additional information regarding copyright ownership.
   The ASF licenses this file to You under the Apache License, Version 2.0
   (the "License"); you may not use this file except in compliance with
   the License.  You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */

package org.apache.poi.ss.util;

import java.util.HashMap;
import java.util.Map;

import org.apache.poi.ss.SpreadsheetVersion;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;

/**
 * <p>
 * A <tt>CellBorder</tt> is a template that can be applied to any sheet in a
 * project. It contains all the border type and color attributes needed to draw
 * all the borders for a single sheet. That template can be applied to any sheet
 * in any workbook.
 * 
 * This class requires the full spreadsheet to be in memory so SXSSFWorkbook 
 * Spreadsheets are not supported. The same <tt>CellBorder</tt> can, however, be
 * applied to both HSSFWorkbook, and XSSFWorkbook objects if necessary. Portions of
 * the border that fall outside the max range of the HSSFWorkbook sheet are ignored.
 * </p>
 * 
 * <p>
 * This would replace <tt>RegionUtils</tt>.
 * </p>
 */
public final class CellBorder {

    /**
     * no border
     */
    public static final short EXTENT_NONE = 0x0;

    /**
     * all vertical and horizontal borders
     */
    public static final short EXTENT_ALL = 0x1;

    /**
     * inside borders
     */
    public static final short EXTENT_INSIDE = 0x2;

    /**
     * outside borders
     */
    public static final short EXTENT_OUTSIDE = 0x3;

    /**
     * top border
     */
    public static final short EXTENT_TOP = 0x4;

    /**
     * bottom border
     */
    public static final short EXTENT_BOTTOM = 0x5;

    /**
     * left border
     */
    public static final short EXTENT_LEFT = 0x6;

    /**
     * right border
     */
    public static final short EXTENT_RIGHT = 0x7;

    /**
     * all horizontal borders
     */
    public static final short EXTENT_HORIZONTAL = 0x8;

    /**
     * inside horizontal borders
     */
    public static final short EXTENT_INSIDE_HORIZONTAL = 0x9;

    /**
     * outside horizontal borders
     */
    public static final short EXTENT_OUTSIDE_HORIZONTAL = 0xA;

    /**
     * all vertical borders
     */
    public static final short EXTENT_VERTICAL = 0xB;

    /**
     * inside vertical borders
     */
    public static final short EXTENT_INSIDE_VERTICAL = 0xC;

    /**
     * outside vertical borders
     */
    public static final short EXTENT_OUTSIDE_VERTICAL = 0xD;

    /**
     * This is a list of cell border properties for one shot application to a
     * group of cells at a later time.
     */
    private Map<CellAddress, Map<String, Object>> _cellBorder;

    /**
     *
     */
    public CellBorder() {
        this._cellBorder = new HashMap<CellAddress, Map<String, Object>>();
    }

    /**
     * <p>
     * Draws a group of cell borders for a cell range. The borders are not
     * applied to the cells at this time, just the template is drawn. To apply
     * the drawn borders to a sheet, use <tt>applyBorders()</tt>.
     * </p>
     * 
     * @param range
     *            - range of cells to draw borders for.
     * @param borderType
     *            - type of border to draw. Use CellStyle.BORDER_XXX constants.
     * @param extent
     *            - the extent of the borders to be drawn. Use
     *            CellBorder.EXTENT_XXX constants.
     */
    public void drawBorders(CellRangeAddress range, short borderType,
            short extent) {
        switch (extent) {
        case EXTENT_NONE:
            break;
        case EXTENT_ALL:
            drawHorizontalBorders(range, borderType, EXTENT_ALL);
            drawVerticalBorders(range, borderType, EXTENT_ALL);
            break;
        case EXTENT_INSIDE:
            drawHorizontalBorders(range, borderType, EXTENT_INSIDE);
            drawVerticalBorders(range, borderType, EXTENT_INSIDE);
            break;
        case EXTENT_OUTSIDE:
            drawOutsideBorders(range, borderType, EXTENT_ALL);
            break;
        case EXTENT_TOP:
            drawTopBorder(range, borderType);
            break;
        case EXTENT_BOTTOM:
            drawBottomBorder(range, borderType);
            break;
        case EXTENT_LEFT:
            drawLeftBorder(range, borderType);
            break;
        case EXTENT_RIGHT:
            drawRightBorder(range, borderType);
            break;
        case EXTENT_HORIZONTAL:
            drawHorizontalBorders(range, borderType, EXTENT_ALL);
            break;
        case EXTENT_INSIDE_HORIZONTAL:
            drawHorizontalBorders(range, borderType, EXTENT_INSIDE);
            break;
        case EXTENT_OUTSIDE_HORIZONTAL:
            drawOutsideBorders(range, borderType, EXTENT_HORIZONTAL);
            break;
        case EXTENT_VERTICAL:
            drawVerticalBorders(range, borderType, EXTENT_ALL);
            break;
        case EXTENT_INSIDE_VERTICAL:
            drawVerticalBorders(range, borderType, EXTENT_INSIDE);
            break;
        case EXTENT_OUTSIDE_VERTICAL:
            drawOutsideBorders(range, borderType, EXTENT_VERTICAL);
            break;
        }
    }

    /**
     * <p>
     * Draws a group of cell borders for a cell range. The borders are not
     * applied to the cells at this time, just the template is drawn. To apply
     * the drawn borders to a sheet, use <tt>applyBorders()</tt>.
     * </p>
     * 
     * @param range
     *            - range of cells to draw borders for.
     * @param borderType
     *            - type of border to draw. Use CellStyle.BORDER_XXX constants.
     * @param color
     *            - the color to use to draw the borders.
     * @param extent
     *            - the extent of the borders to be drawn. Use
     *            CellBorder.EXTENT_XXX constants.
     */
    public void drawBorders(CellRangeAddress range, short borderType,
            short color, short extent) {
        drawBorders(range, borderType, extent);
        if (borderType != CellStyle.BORDER_NONE) {
            drawBorderColors(range, color, extent);
        }
    }

    /**
     * <p>
     * Draws the top border for a range of cells
     * </p>
     * 
     * @param range
     * @param borderType
     */
    private void drawTopBorder(CellRangeAddress range, short borderType) {
        int row = range.getFirstRow();
        int firstCol = range.getFirstColumn();
        int lastCol = range.getLastColumn();
        for (int i = firstCol; i <= lastCol; i++) {
            addProperty(row, i, CellUtil.BORDER_TOP, borderType);
            if (borderType == CellStyle.BORDER_NONE && row > 0) {
                addProperty(row - 1, i, CellUtil.BORDER_BOTTOM, borderType);
            }
        }
    }

    /**
     * <p>
     * Draws the bottom border for a range of cells
     * </p>
     * 
     * @param range
     * @param borderType
     */
    private void drawBottomBorder(CellRangeAddress range, short borderType) {
        int row = range.getLastRow();
        int firstCol = range.getFirstColumn();
        int lastCol = range.getLastColumn();
        for (int i = firstCol; i <= lastCol; i++) {
            addProperty(row, i, CellUtil.BORDER_BOTTOM, borderType);
            if (borderType == CellStyle.BORDER_NONE
                    && row < SpreadsheetVersion.EXCEL2007.getMaxRows() - 1) {
                addProperty(row + 1, i, CellUtil.BORDER_TOP, borderType);
            }
        }
    }

    /**
     * <p>
     * Draws the left border for a range of cells
     * </p>
     *
     * @param range
     * @param borderType
     */
    private void drawLeftBorder(CellRangeAddress range, short borderType) {
        int firstRow = range.getFirstRow();
        int lastRow = range.getLastRow();
        int col = range.getFirstColumn();
        for (int i = firstRow; i <= lastRow; i++) {
            addProperty(i, col, CellUtil.BORDER_LEFT, borderType);
            if (borderType == CellStyle.BORDER_NONE && col > 0) {
                addProperty(i, col - 1, CellUtil.BORDER_RIGHT, borderType);
            }
        }
    }

    /**
     * <p>
     * Draws the right border for a range of cells
     * </p>
     *
     * @param range
     * @param borderType
     */
    private void drawRightBorder(CellRangeAddress range, short borderType) {
        int firstRow = range.getFirstRow();
        int lastRow = range.getLastRow();
        int col = range.getLastColumn();
        for (int i = firstRow; i <= lastRow; i++) {
            addProperty(i, col, CellUtil.BORDER_RIGHT, borderType);
            if (borderType == CellStyle.BORDER_NONE
                    && col < SpreadsheetVersion.EXCEL2007.getMaxColumns() - 1) {
                addProperty(i, col + 1, CellUtil.BORDER_LEFT, borderType);
            }
        }
    }

    /**
     * <p>
     * Draws the outside borders for a range of cells.
     * </p>
     *
     * @param range
     * @param borderType
     * @param extent
     */
    private void drawOutsideBorders(CellRangeAddress range, short borderType,
            short extent) {
        if (extent == EXTENT_ALL || extent == EXTENT_HORIZONTAL) {
            drawTopBorder(range, borderType);
            drawBottomBorder(range, borderType);
        }
        if (extent == EXTENT_ALL || extent == EXTENT_VERTICAL) {
            drawLeftBorder(range, borderType);
            drawRightBorder(range, borderType);
        }
    }

    /**
     * <p>
     * Draws the horizontal borders for a range of cells.
     * </p>
     *
     * @param range
     * @param borderType
     * @param extent
     */
    private void drawHorizontalBorders(CellRangeAddress range, short borderType,
            short extent) {
        int firstRow = range.getFirstRow();
        int lastRow = range.getLastRow();
        int firstCol = range.getFirstColumn();
        int lastCol = range.getLastColumn();
        for (int i = firstRow; i <= lastRow; i++) {
            CellRangeAddress row = new CellRangeAddress(i, i, firstCol,
                    lastCol);
            if (extent == EXTENT_ALL || i > firstRow) {
                drawTopBorder(row, borderType);
            }
            if (extent == EXTENT_ALL || i < lastRow) {
                drawBottomBorder(row, borderType);
            }
        }
    }

    /**
     * <p>
     * Draws the vertical borders for a range of cells.
     * </p>
     *
     * @param range
     * @param borderType
     * @param extent
     */
    private void drawVerticalBorders(CellRangeAddress range, short borderType,
            short extent) {
        int firstRow = range.getFirstRow();
        int lastRow = range.getLastRow();
        int firstCol = range.getFirstColumn();
        int lastCol = range.getLastColumn();
        for (int i = firstCol; i <= lastCol; i++) {
            CellRangeAddress row = new CellRangeAddress(firstRow, lastRow, i,
                    i);
            if (extent == EXTENT_ALL || i > firstCol) {
                drawLeftBorder(row, borderType);
            }
            if (extent == EXTENT_ALL || i < lastCol) {
                drawRightBorder(row, borderType);
            }
        }
    }

    /**
     * <p>
     * Applies the drawn borders to a Sheet. The borders that are applied are
     * the ones that have been drawn by the <tt>drawBorders</tt> and
     * <tt>drawBorderColors</tt> methods.
     * </p>
     *
     * @param sheet
     *            - the sheet on which to apply borders
     */
    public void applyBorders(Sheet sheet) {
        Workbook wb = sheet.getWorkbook();
        for (Map.Entry<CellAddress, Map<String, Object>> entry : this._cellBorder
                .entrySet()) {
            CellAddress cellAddress = entry.getKey();
            if (cellAddress.getRow() < wb.getSpreadsheetVersion().getMaxRows()
                    && cellAddress.getColumn() < wb.getSpreadsheetVersion().getMaxColumns()) {
                Map<String, Object> properties = entry.getValue();
                Row row = CellUtil.getRow(cellAddress.getRow(), sheet);
                Cell cell = CellUtil.getCell(row, cellAddress.getColumn());
                CellUtil.setCellStyleProperties(cell, properties);
            }
        }
    }

    /**
     * <p>
     * Sets the color for borders for a range of cells.
     * </p>
     *
     * @param range
     *            - The range of cells to set the color on
     * @param color
     *            - The color
     * @param extent
     *            - The extent of the borders for which colors are set
     */
    public void drawBorderColors(CellRangeAddress range, short color,
            short extent) {
        switch (extent) {
        case EXTENT_NONE:
            break;
        case EXTENT_ALL:
            drawHorizontalBorderColors(range, color, EXTENT_ALL);
            drawVerticalBorderColors(range, color, EXTENT_ALL);
            break;
        case EXTENT_INSIDE:
            drawHorizontalBorderColors(range, color, EXTENT_INSIDE);
            drawVerticalBorderColors(range, color, EXTENT_INSIDE);
            break;
        case EXTENT_OUTSIDE:
            drawOutsideBorderColors(range, color, EXTENT_ALL);
            break;
        case EXTENT_TOP:
            drawTopBorderColor(range, color);
            break;
        case EXTENT_BOTTOM:
            drawBottomBorderColor(range, color);
            break;
        case EXTENT_LEFT:
            drawLeftBorderColor(range, color);
            break;
        case EXTENT_RIGHT:
            drawRightBorderColor(range, color);
            break;
        case EXTENT_HORIZONTAL:
            drawHorizontalBorderColors(range, color, EXTENT_ALL);
            break;
        case EXTENT_INSIDE_HORIZONTAL:
            drawHorizontalBorderColors(range, color, EXTENT_INSIDE);
            break;
        case EXTENT_OUTSIDE_HORIZONTAL:
            drawOutsideBorderColors(range, color, EXTENT_HORIZONTAL);
            break;
        case EXTENT_VERTICAL:
            drawVerticalBorderColors(range, color, EXTENT_ALL);
            break;
        case EXTENT_INSIDE_VERTICAL:
            drawVerticalBorderColors(range, color, EXTENT_INSIDE);
            break;
        case EXTENT_OUTSIDE_VERTICAL:
            drawOutsideBorderColors(range, color, EXTENT_VERTICAL);
            break;
        }
    }

    /**
     * <p>
     * Sets the color of the top border for a range of cells.
     * </p>
     *
     * @param range
     * @param color
     */
    private void drawTopBorderColor(CellRangeAddress range, short color) {
        int row = range.getFirstRow();
        int firstCol = range.getFirstColumn();
        int lastCol = range.getLastColumn();
        for (int i = firstCol; i <= lastCol; i++) {
            addProperty(row, i, CellUtil.TOP_BORDER_COLOR, color);
        }
    }

    /**
     * <p>
     * Sets the color of the bottom border for a range of cells.
     * </p>
     *
     * @param range
     * @param color
     */
    private void drawBottomBorderColor(CellRangeAddress range, short color) {
        int row = range.getLastRow();
        int firstCol = range.getFirstColumn();
        int lastCol = range.getLastColumn();
        for (int i = firstCol; i <= lastCol; i++) {
            addProperty(row, i, CellUtil.BOTTOM_BORDER_COLOR, color);
        }
    }

    /**
     * <p>
     * Sets the color of the left border for a range of cells.
     * </p>
     *
     * @param range
     * @param color
     */
    private void drawLeftBorderColor(CellRangeAddress range, short color) {
        int firstRow = range.getFirstRow();
        int lastRow = range.getLastRow();
        int col = range.getFirstColumn();
        for (int i = firstRow; i <= lastRow; i++) {
            addProperty(1, col, CellUtil.LEFT_BORDER_COLOR, color);
        }
    }

    /**
     * <p>
     * Sets the color of the right border for a range of cells.
     * </p>
     *
     * @param range
     * @param color
     */
    private void drawRightBorderColor(CellRangeAddress range, short color) {
        int firstRow = range.getFirstRow();
        int lastRow = range.getLastRow();
        int col = range.getLastColumn();
        for (int i = firstRow; i <= lastRow; i++) {
            addProperty(i, col, CellUtil.RIGHT_BORDER_COLOR, color);
        }
    }

    /**
     * <p>
     * Sets the color of the outside borders for a range of cells.
     * </p>
     *
     * @param range
     * @param color
     * @param extent
     */
    private void drawOutsideBorderColors(CellRangeAddress range, short color,
            short extent) {
        if (extent == EXTENT_ALL || extent == EXTENT_HORIZONTAL) {
            drawTopBorderColor(range, color);
            drawBottomBorderColor(range, color);
        }
        if (extent == EXTENT_ALL || extent == EXTENT_VERTICAL) {
            drawLeftBorderColor(range, color);
            drawRightBorderColor(range, color);
        }
    }

    /**
     * <p>
     * Sets the color of the horizontal borders for a range of cells.
     * </p>
     *
     * @param range
     * @param color
     * @param extent
     */
    private void drawHorizontalBorderColors(CellRangeAddress range, short color,
            short extent) {
        int firstRow = range.getFirstRow();
        int lastRow = range.getLastRow();
        int firstCol = range.getFirstColumn();
        int lastCol = range.getLastColumn();
        for (int i = firstRow; i <= lastRow; i++) {
            CellRangeAddress row = new CellRangeAddress(i, i, firstCol,
                    lastCol);
            if (extent == EXTENT_ALL || i > firstRow) {
                drawTopBorderColor(row, color);
            }
            if (extent == EXTENT_ALL || i < lastRow) {
                drawBottomBorderColor(row, color);
            }
        }
    }

    /**
     * <p>
     * Sets the color of the vertical borders for a range of cells.
     * </p>
     *
     * @param range
     * @param color
     * @param extent
     */
    private void drawVerticalBorderColors(CellRangeAddress range, short color,
            short extent) {
        int firstRow = range.getFirstRow();
        int lastRow = range.getLastRow();
        int firstCol = range.getFirstColumn();
        int lastCol = range.getLastColumn();
        for (int i = firstCol; i <= lastCol; i++) {
            CellRangeAddress row = new CellRangeAddress(firstRow, lastRow, i,
                    i);
            if (extent == EXTENT_ALL || i > firstCol) {
                drawLeftBorderColor(row, color);
            }
            if (extent == EXTENT_ALL || i < lastCol) {
                drawRightBorderColor(row, color);
            }
        }
    }

    /**
     * Adds a property to the border Map for a given cell
     *
     * @param cell
     * @param property
     * @param value
     */
    private void addProperty(CellAddress cell, String property, Object value) {
        Map<String, Object> cellProperties = this._cellBorder.get(cell);
        if (cellProperties == null) {
            cellProperties = new HashMap<String, Object>();
        }
        cellProperties.put(property, value);
        this._cellBorder.put(cell, cellProperties);
    }

    /**
     * Adds a property to the border Map for a given cell
     *
     * @param cell
     * @param property
     * @param value
     */
    private void addProperty(int row, int col, String property, Object value) {
        addProperty(new CellAddress(row, col), property, value);
    }
}
